// @ myth - common - string.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/string.h"

// @@ string_link

// @@@ string_link::string_link(const char* s, int l, string_type t)

string_link::string_link(const char* s, int l, string_type t)
{
    string = (t == STRING_STATIC) ? s : strcpy(new char[l + 1], s);
    length = l;
    count = 1;
    type = t;
}

// @@@ string_link::~string_link()

string_link::~string_link()
{
    if(type == STRING_COPY)
	delete string;
}

// @@ STRING

// @@@ void STRING::UnLink()

void STRING::UnLink()
{
    if(link != NULL && --(link->count) < 1)
	delete link;
    
    link = NULL;
}

// @@@ void STRING::Link(const STRING* copy)

void STRING::Link(const STRING* copy)
{
    if((link = copy->link) != NULL)
	(link->count)++;
}

// @@@ const STRING& STRING::operator=(const STRING& copy)

const STRING& STRING::operator=(const STRING& copy)
{
    UnLink();
    Link(&copy);
    return copy;
}

// @@@ bool STRING::operator==(const STRING& other)

bool STRING::operator==(const STRING& other)
{
    if(link == other.link)
	return TRUE;
    else if(link == NULL || other.link == NULL)
	return FALSE;
    else if(link->string == other.link->string)
	return TRUE;
    else if(link->length != other.link->length)
	return FALSE;
    else if(strcmp(link->string, other.link->string) == EQUAL)
	return TRUE;
    else
	return FALSE;
}

// @@@ bool STRING::operator!=(const STRING& other)

bool STRING::operator!=(const STRING& other)
{
    if(link == other.link)
	return FALSE;
    else if(link == NULL || other.link == NULL)
	return TRUE;
    else if(link->string == other.link->string)
	return FALSE;
    else if(link->length != other.link->length)
	return TRUE;
    else if(strcmp(link->string, other.link->string) == EQUAL)
	return FALSE;
    else
	return TRUE;
}

// @@@ STRING::STRING(const char* string, string_type type = STRING_COPY)

STRING::STRING(const char* string, string_type type = STRING_COPY)
{ link = new string_link(string, strlen(string), type); }

// @@@ STRING::STRING(const STRING& copy)

STRING::STRING(const STRING& copy)
{ Link(&copy); }

// @@@ STRING::STRING()

STRING::STRING()
{ link = NULL; }

// @@@ STRING::~STRING()

STRING::~STRING()
{ UnLink(); }

// @@ STRONG

// @@@ STRONG::STRONG(const char* string)

STRONG::STRONG(const char* string) : (string, STRING_STATIC)
{ }

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
